@@ -7,11 +7,18 @@
    
 #include <gdk/gdk.h>
 #include <gtk/gtk.h>
+
+#include "PerlGtkInt.h"
 #include "MiscTypes.h"
 #include "GdkTypes.h"
 #include "GtkTypes.h"
 #include "GtkDefs.h"
 
+SV * newSVGdkBitmap(GdkBitmap * x) { return newSVGdkWindow(x); }
+GdkBitmap * SvGdkBitmap(SV * x) { return SvGdkWindow(x); }
+SV * newSVGdkPixmap(GdkPixmap * x) { return newSVGdkWindow(x); }
+GdkPixmap * SvGdkPixmap(SV * x) { return SvGdkWindow(x); }
+
 /*SV * newSVGdkWindowRef(GdkWindow * w) { return newSVMiscRef(w, "Gtk::Gdk::Window",0); }
 GdkWindow * SvGdkWindowRef(SV * data) { return SvMiscRef(data, "Gtk::Gdk::Window"); }
 
