@@ -1,6 +1,13 @@
 
 /* Copyright (C) 1997, Kenneth Albanowski.
    This code may be distributed under the same terms as Perl itself. */
+
+#ifndef _Gdk_Types_h_
+#define _Gdk_Types_h_
+
+#ifndef PerlGtkDeclareFunc
+#include "PerlGtkInt.h"
+#endif
    
 
 /*typedef GdkPixmap * Gtk__Gdk__Pixmap;
@@ -40,20 +47,20 @@ typedef GdkGCValues Gtk__Gdk__Values;
 /*typedef GdkInputMode Gtk__Gdk__InputMode;*/
 /*typedef GdkAxisUse Gtk__Gdk__AxisUse;*/
 
-extern int SvGdkEventType(SV * value);
-extern SV * newSVGdkEventType(int value);
-extern int SvGdkModifierType(SV * value);
-extern SV * newSVGdkModifierType(int value);
-extern int SvGdkGCValuesMask(SV * value);
-extern SV * newSVGdkGCValuesMask(int value);
-extern SV * newSVGdkGCValues(GdkGCValues * v);
-extern GdkGCValues * SvGdkGCValues(SV * data, GdkGCValues * v, GdkGCValuesMask * m);
-extern int SvGdkWindowType(SV * value);
-extern SV * newSVGdkWindowType(int value);
-extern int SvGdkNotifyType(SV * value);
-extern SV * newSVGdkNotifyType(int value);
-extern int SvGdkVisualType(SV * value);
-extern SV * newSVGdkVisualType(int value);
+/*extern int SvGdkEventType(SV * value);
+extern SV * newSVGdkEventType(int value);*/
+/*extern int SvGdkModifierType(SV * value);
+extern SV * newSVGdkModifierType(int value);*/
+/*PxerlGtkFunc(int, SvGdkGCValuesMask)(SV * value);
+PxerlGtkFunc(SV *, newSVGdkGCValuesMask)(int value);*/
+PerlGtkDeclareFunc(SV *, newSVGdkGCValues)(GdkGCValues * v);
+PerlGtkDeclareFunc(GdkGCValues *, SvGdkGCValues)(SV * data, GdkGCValues * v, GdkGCValuesMask * m);
+/* extern int SvGdkWindowType(SV * value); */
+/* extern SV * newSVGdkWindowType(int value); */
+/*PxerlGtkFunc(int, SvGdkNotifyType)(SV * value);
+PxerlGtkFunc(SV *, newSVGdkNotifyType)(int value);*/
+/*PxerlGtkFunc(int, SvGdkVisualType)(SV * value);
+PxerlGtkFunc(SV *, newSVGdkVisualType)(int value);*/
 
 /*extern int SvGdkInputMode(SV * value);
 extern SV * newSVGdkInputMode(int value);*/
@@ -62,31 +69,31 @@ extern SV * newSVGdkInputSource(int value);*/
 /*extern int SvGdkAxisUse(SV * value);
 extern SV * newSVGdkAxisUse(int value);*/
 
-extern SV * newSVGdkDeviceInfo(GdkDeviceInfo * i);
-extern SV * newSVGdkTimeCoord(GdkTimeCoord * i);
+PerlGtkDeclareFunc(SV *, newSVGdkDeviceInfo)(GdkDeviceInfo * i);
+PerlGtkDeclareFunc(SV *, newSVGdkTimeCoord)(GdkTimeCoord * i);
 
-extern int SvGdkSubwindowMode(SV * value);
-extern SV * newSVGdkSubwindowMode(int value);
+/*PxerlGtkFunc(int, SvGdkSubwindowMode)(SV * value);
+PxerlGtkFunc(SV *, newSVGdkSubwindowMode)(int value);*/
 /*extern int SvGdkLineStyle(SV * value);
 extern SV * newSVGdkLineStyle(int value);*/
-extern int SvGdkFunction(SV * value);
-extern SV * newSVGdkFunction(int value);
-extern int SvGdkJoinStyle(SV * value);
-extern SV * newSVGdkJoinStyle(int value);
+/*extern int SvGdkFunction(SV * value);
+extern SV * newSVGdkFunction(int value);*/
+/*extern int SvGdkJoinStyle(SV * value);*/
+/*PxerlGtkFunc(SV *, newSVGdkJoinStyle)(int value);*/
 /*extern int SvGdkFill(SV * value);
 extern SV * newSVGdkFill(int value);*/
-extern int SvGdkCapStyle(SV * value);
-extern SV * newSVGdkCapStyle(int value);
-extern int SvGdkWindowClass(SV * value);
-extern SV * newSVGdkWindowClass(int value);
-extern int SvGdkWindowType(SV * value);
-extern SV * newSVGdkWindowType(int value);
-extern int SvGdkCursorType(SV * value);
-extern SV * newSVGdkCursorType(int value);
+/*PxerlGtkFunc(int, SvGdkCapStyle)(SV * value);
+PxerlGtkFunc(SV *, newSVGdkCapStyle)(int value);*/
+/*PxerlGtkFunc(int, SvGdkWindowClass)(SV * value);
+PxerlGtkFunc(SV *, newSVGdkWindowClass)(int value);*/
+/*PxerlGtkFunc(int, SvGdkWindowType)(SV * value);
+PxerlGtkFunc(SV *, newSVGdkWindowType)(int value);*/
+/*PxerlGtkFunc(int, SvGdkCursorType)(SV * value);
+PxerlGtkFunc(SV *, newSVGdkCursorType)(int value);*/
 /*extern int SvGdkEventMask(SV * value);
 extern SV * newSVGdkEventMask(int value);*/
-extern int SvGdkInputCondition(SV * value);
-extern SV * newSVGdkInputCondition(int value);
+/*PxerlGtkFunc(int, SvGdkInputCondition)(SV * value);
+PxerlGtkFunc(SV *, newSVGdkInputCondition)(int value);*/
 /*extern SV * newSVGdkWindowRef(GdkWindow * w);
 extern GdkWindow * SvGdkWindowRef(SV * data);
 extern SV * newSVGdkPixmapRef(GdkPixmap * w);
@@ -105,24 +112,27 @@ extern SV * newSVGdkFontRef(GdkFont * f);
 extern GdkFont * SvGdkFontRef(SV * data);
 extern SV * newSVGdkImageRef(GdkImage * i);
 extern GdkImage * SvGdkImageRef(SV * data);*/
-extern SV * newSVGdkRectangle(GdkRectangle * rect);
-extern GdkRectangle * SvGdkRectangle(SV * data, GdkRectangle * rect);
+PerlGtkDeclareFunc(SV *, newSVGdkRectangle)(GdkRectangle * rect);
+PerlGtkDeclareFunc(GdkRectangle *, SvGdkRectangle)(SV * data, GdkRectangle * rect);
 /*extern SV * newSVGdkColor(GdkColor * color);
 extern GdkColor * SvGdkColor(SV * data);*/
-extern SV * newSVGdkAtom(GdkAtom a);
-extern GdkAtom SvGdkAtom(SV * data);
-extern SV * newSVGdkEvent(GdkEvent * e);
-extern GdkEvent * SvGdkEvent(SV * data);
-extern SV * newSVGdkRegion(GdkRegion * e);
-extern GdkRegion * SvGdkRegion(SV * data);
-extern GdkWindowAttr * SvGdkWindowAttr(SV * data, GdkWindowAttr * attr, gint * mask);
+PerlGtkDeclareFunc(SV *, newSVGdkAtom)(GdkAtom a);
+PerlGtkDeclareFunc(GdkAtom, SvGdkAtom)(SV * data);
+/*extern SV * newSVGdkEvent(GdkEvent * e);
+extern GdkEvent * SvGdkEvent(SV * data);*/
+PerlGtkDeclareFunc(SV *, newSVGdkRegion)(GdkRegion * e);
+PerlGtkDeclareFunc(GdkRegion *, SvGdkRegion)(SV * data);
+PerlGtkDeclareFunc(GdkWindowAttr *, SvGdkWindowAttr)(SV * data, GdkWindowAttr * attr, gint * mask);
 
 #define newSVGdkImage(data) newSVMiscRef((void*)data, "Gtk::Gdk::Image", 0)
 #define SvGdkImage(data) (GdkImage*)SvMiscRef(data,0)
 
-extern SV * newSVGdkWindow(GdkWindow * value);
-extern GdkWindow * SvGdkWindow(SV * value);
-#define newSVGdkBitmap(x) newSVGdkWindow(x)
+PerlGtkDeclareFunc(SV *, newSVGdkWindow)(GdkWindow * value);
+PerlGtkDeclareFunc(GdkWindow *, SvGdkWindow)(SV * value);
+/*#define newSVGdkBitmap(x) newSVGdkWindow(x)
 #define SvGdkBitmap(x) SvGdkWindow(x)
 #define newSVGdkPixmap(x) newSVGdkWindow(x)
-#define SvGdkPixmap(x) SvGdkWindow(x)
+#define SvGdkPixmap(x) SvGdkWindow(x)*/
+
+#endif /*_Gdk_Types_h_*/
+
