@@ -6,7 +6,7 @@ require AutoLoader;
 
 use Carp;
 
-$VERSION = '0.3';
+$VERSION = '0.4';
 
 @ISA = qw(Exporter DynaLoader);
 # Items to export into callers namespace by default. Note: do not export
@@ -59,10 +59,6 @@ require Gtk::Types;
 
 $Gtk::_init_package = "Gtk" if not defined $Gtk::_init_package;
 
-if ((defined $Gtk::_init_package) && ($Gtk::_init_package ne "none")) {
-	$Gtk::_init_package->init;
-}
-
 package Gtk::Object;
 
 use Carp;
