@@ -6,14 +6,41 @@
    This code may be distributed under the same terms as Perl itself. */
    
 #include <gtk/gtk.h>
+#include "PerlGtkInt.h"
 #include "GtkTypes.h"
 #include "GdkTypes.h"
 #include "MiscTypes.h"
 
+#include "GtkDefs.h"
+
 static HV * ObjectCache = 0;
 
 #define TRY_MM
 
+AV * gtk_typecasts = 0;
+static HV * types = 0;
+
+void add_typecast(int type, char * perlName)
+{
+	if (!gtk_typecasts)
+		gtk_typecasts = newAV();
+	if (!types)
+		types = newHV();
+	/*GtkObjectClass * klass = gtk_type_class(type);*/
+	av_extend(gtk_typecasts, type/* klass->type*/);
+	av_store(gtk_typecasts, type/*klass->type*/, newSVpv(perlName, 0));
+	hv_store(types, perlName, strlen(perlName), newSViv(type), 0);
+}
+
+int type_name(char * name) {
+	SV ** s = hv_fetch(types, name, strlen(name), 0);
+	if (s)
+		return SvIV(*s);
+	else
+		return 0;
+}
+
+
 void UnregisterGtkObject(SV * sv_object, GtkObject * gtk_object)
 {
 	char buffer[40];
@@ -109,8 +136,6 @@ int GCGtkObjects(void) {
     return 0;
 }
 
-extern AV * gtk_typecasts;
-
 int gc_during_idle = 0;
 
 static void GCDuringIdle(void);
@@ -579,3 +604,307 @@ GtkType FindArgumentType(GtkObject * object, SV * name, GtkArg * result) {
 	
 	return t;
 }
+
+
+struct PerlGtkTypeHelper * PerlGtkTypeHelpers = 0;
+
+void AddTypeHelper(struct PerlGtkTypeHelper * n)
+{
+	struct PerlGtkTypeHelper * h = PerlGtkTypeHelpers;
+	
+	if (!n)
+		return;
+
+	n->next = 0;
+	if (!h) {
+		PerlGtkTypeHelpers = n;
+		return;
+	}
+	
+	while (h->next)
+		h = h->next;
+
+	h->next = n;
+}
+
+SV * GtkGetArg(GtkArg * a)
+{
+	SV * result = 0;
+	switch (GTK_FUNDAMENTAL_TYPE(a->type)) {
+		case GTK_TYPE_CHAR:	result = newSViv(GTK_VALUE_CHAR(*a)); break;
+		case GTK_TYPE_BOOL:	result = newSViv(GTK_VALUE_BOOL(*a)); break;
+		case GTK_TYPE_INT:	result = newSViv(GTK_VALUE_INT(*a)); break;
+		case GTK_TYPE_UINT:	result = newSViv(GTK_VALUE_UINT(*a)); break;
+		case GTK_TYPE_LONG:	result = newSViv(GTK_VALUE_LONG(*a)); break;
+		case GTK_TYPE_ULONG:	result = newSViv(GTK_VALUE_ULONG(*a)); break;
+		case GTK_TYPE_FLOAT:	result = newSVnv(GTK_VALUE_FLOAT(*a)); break;	
+		case GTK_TYPE_DOUBLE:	result = newSVnv(GTK_VALUE_DOUBLE(*a)); break;	
+		case GTK_TYPE_STRING:	result = GTK_VALUE_STRING(*a) ? newSVpv(GTK_VALUE_STRING(*a),0) : newSVsv(&sv_undef); break;
+		case GTK_TYPE_OBJECT:	result = newSVGtkObjectRef(GTK_VALUE_OBJECT(*a), 0); break;
+		case GTK_TYPE_SIGNAL:
+		{
+			AV * args = (AV*)GTK_VALUE_SIGNAL(*a).d;
+			SV ** s;
+			if ((GTK_VALUE_SIGNAL(*a).f != 0) ||
+				(!args) ||
+				(SvTYPE(args) != SVt_PVAV) ||
+				(av_len(args) < 3) ||
+				!(s = av_fetch(args, 2, 0))
+				)
+				croak("Unable to return a foreign signal type to Perl");
+
+			result = newSVsv(*s);
+			break;
+		}
+		case GTK_TYPE_ENUM:
+			break;
+		case GTK_TYPE_FLAGS:
+			break;
+		case GTK_TYPE_POINTER:
+#if 0
+			if (a->type == GTK_TYPE_POINTER_CHAR)
+				result = newSViv(*GTK_RETLOC_CHAR(*a));
+			else
+			if (a->type == GTK_TYPE_POINTER_BOOL)
+				result = newSViv(*GTK_RETLOC_BOOL(*a));
+			else
+			if (a->type == GTK_TYPE_POINTER_INT)
+				result = newSViv(*GTK_RETLOC_INT(*a));
+			else
+			if (a->type == GTK_TYPE_POINTER_UINT)
+				result = newSViv(*GTK_RETLOC_UINT(*a));
+			else
+			if (a->type == GTK_TYPE_POINTER_LONG)
+				result = newSViv(*GTK_RETLOC_LONG(*a));
+			else
+			if (a->type == GTK_TYPE_POINTER_ULONG)
+				result = newSViv(*GTK_RETLOC_ULONG(*a));
+			else
+			if (a->type == GTK_TYPE_POINTER_FLOAT)
+				result = newSVnv(*GTK_RETLOC_FLOAT(*a));
+			else
+			if (a->type == GTK_TYPE_POINTER_DOUBLE)
+				result = newSVnv(*GTK_RETLOC_DOUBLE(*a));
+			else
+			if (a->type == GTK_TYPE_POINTER_STRING)
+				result = *GTK_RETLOC_STRING(*a) ? newSVpv(*GTK_RETLOC_STRING(*a), 0) : newSVsv(&sv_undef);
+			else
+			if (a->type == GTK_TYPE_POINTER_OBJECT)
+				result = newSVGtkObjectRef(*GTK_RETLOC_OBJECT(*a));
+			else
+#endif
+			break;
+		case GTK_TYPE_BOXED:
+			if (a->type == GTK_TYPE_GDK_EVENT)
+				result = newSVGdkEvent(GTK_VALUE_BOXED(*a));
+			else
+			break;
+	}
+	
+	if (!result) {
+		struct PerlGtkTypeHelper * h = PerlGtkTypeHelpers;
+		while (!result && h && h->GtkGetArg_f) {
+			result = h->GtkGetArg_f(a);
+			h = h->next;
+		}
+		
+	}
+	
+	if (!result)
+		croak("Cannot set argument of type %s (fundamental type %s)", gtk_type_name(a->type), gtk_type_name(GTK_FUNDAMENTAL_TYPE(a->type)));
+
+	return result;
+}
+
+void GtkSetArg(GtkArg * a, SV * v, SV * Class, GtkObject * Object)
+{
+	int result = 1;
+	switch (GTK_FUNDAMENTAL_TYPE(a->type)) {
+		case GTK_TYPE_CHAR:		GTK_VALUE_CHAR(*a) = SvIV(v); break;
+		case GTK_TYPE_BOOL:		GTK_VALUE_BOOL(*a) = SvIV(v); break;
+		case GTK_TYPE_INT:		GTK_VALUE_INT(*a) = SvIV(v); break;
+		case GTK_TYPE_UINT:		GTK_VALUE_UINT(*a) = SvIV(v); break;
+		case GTK_TYPE_LONG:		GTK_VALUE_LONG(*a) = SvIV(v); break;
+		case GTK_TYPE_ULONG:	GTK_VALUE_ULONG(*a) = SvIV(v); break;
+		case GTK_TYPE_FLOAT:	GTK_VALUE_FLOAT(*a) = SvNV(v); break;	
+		case GTK_TYPE_DOUBLE:	GTK_VALUE_DOUBLE(*a) = SvNV(v); break;	
+		case GTK_TYPE_STRING:	GTK_VALUE_STRING(*a) = g_strdup(SvPV(v,na)); break;
+		case GTK_TYPE_OBJECT:	GTK_VALUE_OBJECT(*a) = SvGtkObjectRef(v, "Gtk::Object"); break;
+		case GTK_TYPE_SIGNAL:
+		{
+			AV * args;
+			int i,j;
+			int type;
+			char * c = strchr(a->name, ':');
+			c+=2;
+			c = strchr(c, ':');
+			c += 2;
+			args = newAV();
+
+			type = gtk_signal_lookup(c, Object->klass->type);
+
+			av_push(args, newSVsv(Class));
+			av_push(args, newSVpv(c, 0));
+			av_push(args, newSViv(type));
+			
+			PackCallback(args, v);
+			/*av_push(args, newSVsv(v));*/
+
+			GTK_VALUE_SIGNAL(*a).f = 0;
+			GTK_VALUE_SIGNAL(*a).d = args;
+			break;
+		}
+		case GTK_TYPE_POINTER:
+#if 0
+			if (a->type == GTK_TYPE_POINTER_CHAR)
+				*GTK_RETLOC_CHAR(*a) = SvIV(v);
+			else
+			if (a->type == GTK_TYPE_POINTER_BOOL)
+				*GTK_RETLOC_BOOL(*a) = SvIV(v);
+			else
+			if (a->type == GTK_TYPE_POINTER_INT)
+				*GTK_RETLOC_INT(*a) = SvIV(v);
+			else
+			if (a->type == GTK_TYPE_POINTER_UINT)
+				*GTK_RETLOC_UINT(*a) = SvIV(v);
+			else
+			if (a->type == GTK_TYPE_POINTER_LONG)
+				*GTK_RETLOC_LONG(*a) = SvIV(v);
+			else
+			if (a->type == GTK_TYPE_POINTER_ULONG)
+				*GTK_RETLOC_ULONG(*a) = SvIV(v);
+			else
+			if (a->type == GTK_TYPE_POINTER_FLOAT)
+				*GTK_RETLOC_FLOAT(*a) = SvNV(v);
+			else
+			if (a->type == GTK_TYPE_POINTER_DOUBLE)
+				*GTK_RETLOC_DOUBLE(*a) = SvNV(v);
+			else
+			if (a->type == GTK_TYPE_POINTER_STRING)
+				*GTK_RETLOC_STRING(*a) = SvPV(v, na);
+			else
+			if (a->type == GTK_TYPE_POINTER_OBJECT)
+				*GTK_RETLOC_OBJECT(*a) = SvGtkObjectRef(v, "Gtk::Object");
+			else
+#endif
+			result = 0;
+			break;
+		case GTK_TYPE_ENUM:
+			result = 0;
+			break;
+		case GTK_TYPE_FLAGS:
+			result = 0;
+			break;
+		case GTK_TYPE_BOXED:
+			if (a->type == GTK_TYPE_GDK_EVENT)
+				GTK_VALUE_BOXED(*a) = SvGdkEvent(v);
+			else
+			result = 0;
+			break;
+		default:
+			result = 0;
+	}
+
+	if (!result) {
+		struct PerlGtkTypeHelper * h = PerlGtkTypeHelpers;
+		while (!result && h && h->GtkSetArg_f) {
+			result = h->GtkSetArg_f(a, v, Class, Object);
+			h = h->next;
+		}
+		
+	}
+
+	if (!result)
+		croak("Cannot set argument of type %s (fundamental type %s)", gtk_type_name(a->type), gtk_type_name(GTK_FUNDAMENTAL_TYPE(a->type)));
+}
+
+void GtkSetRetArg(GtkArg * a, SV * v, SV * Class, GtkObject * Object)
+{
+	int result = 1;
+	switch (GTK_FUNDAMENTAL_TYPE(a->type)) {
+		case GTK_TYPE_CHAR:		*GTK_RETLOC_CHAR(*a) = SvIV(v); break;
+		case GTK_TYPE_BOOL:		*GTK_RETLOC_BOOL(*a) = SvIV(v); break;
+		case GTK_TYPE_INT:		*GTK_RETLOC_INT(*a) = SvIV(v); break;
+		case GTK_TYPE_UINT:		*GTK_RETLOC_UINT(*a) = SvIV(v); break;
+		case GTK_TYPE_LONG:		*GTK_RETLOC_LONG(*a) = SvIV(v); break;
+		case GTK_TYPE_ULONG:	*GTK_RETLOC_ULONG(*a) = SvIV(v); break;
+		case GTK_TYPE_FLOAT:	*GTK_RETLOC_FLOAT(*a) = SvNV(v); break;	
+		case GTK_TYPE_DOUBLE:	*GTK_RETLOC_DOUBLE(*a) = SvNV(v); break;	
+		case GTK_TYPE_STRING:	*GTK_RETLOC_STRING(*a) = SvPV(v,na); break;
+		case GTK_TYPE_OBJECT:	*GTK_RETLOC_OBJECT(*a) = SvGtkObjectRef(v, "Gtk::Object"); break;
+		case GTK_TYPE_ENUM:
+			result = 0;
+			break;
+		case GTK_TYPE_FLAGS:
+			result = 0;
+			break;
+		case GTK_TYPE_POINTER:
+			result = 0;
+			break;
+		case GTK_TYPE_BOXED:
+			if (a->type == GTK_TYPE_GDK_EVENT)
+				*GTK_RETLOC_BOXED(*a) = SvGdkEvent(v);
+			else
+				result = 0;
+			break;
+		default:
+			result = 0;
+	}
+	
+	if (!result) {
+		struct PerlGtkTypeHelper * h = PerlGtkTypeHelpers;
+		while (!result && h && h->GtkSetRetArg_f) {
+			result = h->GtkSetRetArg_f(a, v, Class, Object);
+			h = h->next;
+		}
+		
+	}
+
+	if (!result)
+		croak("Cannot set argument of type %s (fundamental type %s)", gtk_type_name(a->type), gtk_type_name(GTK_FUNDAMENTAL_TYPE(a->type)));
+}
+
+SV * GtkGetRetArg(GtkArg * a)
+{
+	SV * result = 0;
+	switch (GTK_FUNDAMENTAL_TYPE(a->type)) {
+		case GTK_TYPE_NONE:		result = newSVsv(&sv_undef); break;
+		case GTK_TYPE_CHAR:		result = newSViv(*GTK_RETLOC_CHAR(*a)); break;
+		case GTK_TYPE_BOOL:		result = newSViv(*GTK_RETLOC_BOOL(*a)); break;
+		case GTK_TYPE_INT:		result = newSViv(*GTK_RETLOC_INT(*a)); break;
+		case GTK_TYPE_UINT:		result = newSViv(*GTK_RETLOC_UINT(*a)); break;
+		case GTK_TYPE_LONG:		result = newSViv(*GTK_RETLOC_LONG(*a)); break;
+		case GTK_TYPE_ULONG:	result = newSViv(*GTK_RETLOC_ULONG(*a)); break;
+		case GTK_TYPE_FLOAT:	result = newSVnv(*GTK_RETLOC_FLOAT(*a)); break;	
+		case GTK_TYPE_DOUBLE:	result = newSVnv(*GTK_RETLOC_DOUBLE(*a)); break;	
+		case GTK_TYPE_STRING:	result = newSVpv(*GTK_RETLOC_STRING(*a),0); break;
+		case GTK_TYPE_OBJECT:	result = newSVGtkObjectRef(GTK_VALUE_OBJECT(*a), 0); break;
+		case GTK_TYPE_ENUM:
+			break;
+		case GTK_TYPE_FLAGS:
+			break;
+		case GTK_TYPE_POINTER:
+			break;
+		case GTK_TYPE_BOXED:
+			if (a->type == GTK_TYPE_GDK_EVENT)
+				result = newSVGdkEvent(*GTK_RETLOC_BOXED(*a));
+			break;			
+		default:
+	}
+	
+	
+	if (!result) {
+		struct PerlGtkTypeHelper * h = PerlGtkTypeHelpers;
+		while (!result && h && h->GtkGetRetArg_f) {
+			result = h->GtkGetRetArg_f(a);
+			h = h->next;
+		}
+		
+	}
+
+	if (!result)
+		croak("Cannot get return argument of type %s (fundamental type %s)", gtk_type_name(a->type), gtk_type_name(GTK_FUNDAMENTAL_TYPE(a->type)));
+	
+	return result;
+}
+
